@@ -12,30 +12,35 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 from __future__ import annotations
+
+from typing import TYPE_CHECKING, Any, Dict
+
 from httpx import AsyncClient
-from typing import TYPE_CHECKING
+
 if TYPE_CHECKING:
     from supertokens_python.supertokens import AppInfo
     from .types import User
     from .interfaces import RecipeInterface, APIInterface
     from typing import Callable, Union, Awaitable
+
 from os import environ
 
 
-def default_get_email_verification_url(app_info: AppInfo):
-    async def func(_: User):
+def default_get_email_verification_url(app_info: AppInfo) -> Callable[[User, Dict[str, Any]], Awaitable[str]]:
+    async def func(_: User, __: Dict[str, Any]):
         return app_info.website_domain.get_as_string_dangerous(
         ) + app_info.website_base_path.get_as_string_dangerous() + '/verify-email'
     return func
 
 
-def default_create_and_send_custom_email(app_info: AppInfo):
-    async def func(user: User, email_verification_url: str):
+def default_create_and_send_custom_email(app_info: AppInfo) -> Callable[[User, str, Dict[str, Any]], Awaitable[None]]:
+    async def func(user: User, email_verification_url: str, _: Dict[str, Any]):
         if ('SUPERTOKENS_ENV' not in environ) or (
                 environ['SUPERTOKENS_ENV'] != 'testing'):
             return
         try:
-            await AsyncClient().post('https://api.supertokens.io/0/st/auth/email/verify', json={'email': user.email, 'appName': app_info.app_name, 'emailVerifyURL': email_verification_url}, headers={'api-version': '0'})
+            async with AsyncClient() as client:
+                await client.post('https://api.supertokens.io/0/st/auth/email/verify', json={'email': user.email, 'appName': app_info.app_name, 'emailVerifyURL': email_verification_url}, headers={'api-version': '0'})  # type: ignore
         except Exception:
             pass
     return func
@@ -48,21 +53,14 @@ class OverrideConfig:
         self.apis = apis
 
 
-class InputEmailVerificationConfig:
-    def __init__(self,
-                 get_email_verification_url: Union[Callable[[User], Awaitable[str]], None] = None,
-                 create_and_send_custom_email: Union[Callable[[User, str], Awaitable[None]], None] = None
-                 ):
-        self.get_email_verification_url = get_email_verification_url
-        self.create_and_send_custom_email = create_and_send_custom_email
-
-
 class ParentRecipeEmailVerificationConfig:
     def __init__(self,
-                 get_email_for_user_id: Callable[[str], Awaitable[str]],
+                 get_email_for_user_id: Callable[[str, Dict[str, Any]], Awaitable[str]],
                  override: Union[OverrideConfig, None] = None,
-                 get_email_verification_url: Union[Callable[[User], Awaitable[str]], None] = None,
-                 create_and_send_custom_email: Union[Callable[[User, str], Awaitable[None]], None] = None
+                 get_email_verification_url: Union[Callable[[
+                     User, Dict[str, Any]], Awaitable[str]], None] = None,
+                 create_and_send_custom_email: Union[Callable[[
+                     User, str, Dict[str, Any]], Awaitable[None]], None] = None
                  ):
         self.override = override
         self.get_email_verification_url = get_email_verification_url
@@ -73,9 +71,9 @@ class ParentRecipeEmailVerificationConfig:
 class EmailVerificationConfig:
     def __init__(self,
                  override: OverrideConfig,
-                 get_email_verification_url: Callable[[User], Awaitable[str]],
-                 create_and_send_custom_email: Callable[[User, str], Awaitable[None]],
-                 get_email_for_user_id: Callable[[str], Awaitable[str]]
+                 get_email_verification_url: Callable[[User, Dict[str, Any]], Awaitable[str]],
+                 create_and_send_custom_email: Callable[[User, str, Dict[str, Any]], Awaitable[None]],
+                 get_email_for_user_id: Callable[[str, Dict[str, Any]], Awaitable[str]]
                  ):
         self.get_email_for_user_id = get_email_for_user_id
         self.get_email_verification_url = get_email_verification_url
@@ -83,7 +81,8 @@ class EmailVerificationConfig:
         self.override = override
 
 
-def validate_and_normalise_user_input(app_info: AppInfo, config: ParentRecipeEmailVerificationConfig):
+def validate_and_normalise_user_input(
+        app_info: AppInfo, config: ParentRecipeEmailVerificationConfig):
     get_email_verification_url = config.get_email_verification_url if config.get_email_verification_url is not None \
         else default_get_email_verification_url(app_info)
     create_and_send_custom_email = config.create_and_send_custom_email if config.create_and_send_custom_email is not None \
