@@ -14,37 +14,32 @@
 from __future__ import annotations
 
 from os import environ
-from typing import List, TYPE_CHECKING, Union
+from typing import TYPE_CHECKING, Any, Dict, List, Union
 
 from supertokens_python.normalised_url_path import NormalisedURLPath
-from supertokens_python.recipe_module import RecipeModule, APIHandled
-from .api.implementation import APIImplementation
-from .interfaces import APIOptions
-from .recipe_implementation import RecipeImplementation
 from supertokens_python.querier import Querier
+from supertokens_python.recipe_module import APIHandled, RecipeModule
+
+from .api.implementation import APIImplementation
+from .interfaces import APIInterface, APIOptions, RecipeInterface
+from .recipe_implementation import RecipeImplementation
 
 if TYPE_CHECKING:
     from supertokens_python.framework.request import BaseRequest
     from supertokens_python.framework.response import BaseResponse
     from supertokens_python.supertokens import AppInfo
     from .utils import SignInAndUpFeature, InputOverrideConfig
-    from supertokens_python.recipe.emailverification.utils import InputEmailVerificationConfig
-from supertokens_python.exceptions import raise_general_exception, SuperTokensError
+
+from supertokens_python.exceptions import (SuperTokensError,
+                                           raise_general_exception)
 from supertokens_python.recipe.emailverification import EmailVerificationRecipe
-from .utils import validate_and_normalise_user_input
-from .api import (
-    handle_sign_in_up_api,
-    handle_authorisation_url_api,
-    handle_apple_redirect_api
-)
-from .constants import (
-    SIGNINUP,
-    AUTHORISATIONURL,
-    APPLE_REDIRECT_HANDLER
-)
-from .exceptions import (
-    SuperTokensThirdPartyError
-)
+
+from .api import (handle_apple_redirect_api, handle_authorisation_url_api,
+                  handle_sign_in_up_api)
+from .constants import APPLE_REDIRECT_HANDLER, AUTHORISATIONURL, SIGNINUP
+from .exceptions import SuperTokensThirdPartyError
+from .utils import (InputEmailVerificationConfig,
+                    validate_and_normalise_user_input)
 
 
 class ThirdPartyRecipe(RecipeModule):
@@ -67,16 +62,15 @@ class ThirdPartyRecipe(RecipeModule):
         self.providers = self.config.sign_in_and_up_feature.providers
         recipe_implementation = RecipeImplementation(
             Querier.get_instance(recipe_id))
-        self.recipe_implementation = recipe_implementation if self.config.override.functions is None else \
+        self.recipe_implementation: RecipeInterface = recipe_implementation if self.config.override.functions is None else \
             self.config.override.functions(recipe_implementation)
         api_implementation = APIImplementation()
-        self.api_implementation = api_implementation if self.config.override.apis is None else \
+        self.api_implementation: APIInterface = api_implementation if self.config.override.apis is None else \
             self.config.override.apis(api_implementation)
 
-    def is_error_from_this_recipe_based_on_instance(self, err):
-        return isinstance(err, SuperTokensError) and (isinstance(err,
-                                                                 SuperTokensThirdPartyError) or self.email_verification_recipe.is_error_from_this_recipe_based_on_instance(
-            err))
+    def is_error_from_this_recipe_based_on_instance(
+            self, err: Exception) -> bool:
+        return isinstance(err, SuperTokensError) and (isinstance(err, SuperTokensThirdPartyError) or self.email_verification_recipe.is_error_from_this_recipe_based_on_instance(err))
 
     def get_apis_handled(self) -> List[APIHandled]:
         return [
@@ -88,34 +82,29 @@ class ThirdPartyRecipe(RecipeModule):
                        self.api_implementation.disable_apple_redirect_handler_post)
         ] + self.email_verification_recipe.get_apis_handled()
 
-    async def handle_api_request(self, request_id: str, request: BaseRequest, path: NormalisedURLPath, method: str,
-                                 response: BaseResponse):
+    async def handle_api_request(self, request_id: str, request: BaseRequest, path: NormalisedURLPath, method: str, response: BaseResponse):
+        api_options = APIOptions(request, response, self.recipe_id, self.config, self.recipe_implementation, self.providers, self.app_info, self.email_verification_recipe.recipe_implementation)
+
         if request_id == SIGNINUP:
             return await handle_sign_in_up_api(self.api_implementation,
-                                               APIOptions(request, response, self.recipe_id, self.config,
-                                                          self.recipe_implementation, self.providers, self.app_info))
-        elif request_id == AUTHORISATIONURL:
+                                               api_options)
+        if request_id == AUTHORISATIONURL:
             return await handle_authorisation_url_api(self.api_implementation,
-                                                      APIOptions(request, response, self.recipe_id, self.config,
-                                                                 self.recipe_implementation, self.providers,
-                                                                 self.app_info))
-        elif request_id == APPLE_REDIRECT_HANDLER:
+                                                      api_options)
+        if request_id == APPLE_REDIRECT_HANDLER:
             return await handle_apple_redirect_api(self.api_implementation,
-                                                   APIOptions(request, response, self.recipe_id, self.config,
-                                                              self.recipe_implementation, self.providers,
-                                                              self.app_info))
-        else:
-            return await self.email_verification_recipe.handle_api_request(request_id, request, path, method, response)
+                                                   api_options)
 
-    async def handle_error(self, request: BaseRequest, error: SuperTokensError, response: BaseResponse):
-        if isinstance(error, SuperTokensThirdPartyError):
-            raise error
-        else:
-            return self.email_verification_recipe.handle_error(
-                request, error, response)
+        return await self.email_verification_recipe.handle_api_request(request_id, request, path, method, response)
+
+    async def handle_error(self, request: BaseRequest, err: SuperTokensError, response: BaseResponse) -> BaseResponse:
+        if isinstance(err, SuperTokensThirdPartyError):
+            raise err
+        return await self.email_verification_recipe.handle_error(
+            request, err, response)
 
     def get_all_cors_headers(self) -> List[str]:
-        return [] + self.email_verification_recipe.get_all_cors_headers()
+        return self.email_verification_recipe.get_all_cors_headers()
 
     @staticmethod
     def init(sign_in_and_up_feature: SignInAndUpFeature,
@@ -126,9 +115,7 @@ class ThirdPartyRecipe(RecipeModule):
                 ThirdPartyRecipe.__instance = ThirdPartyRecipe(
                     ThirdPartyRecipe.recipe_id, app_info, sign_in_and_up_feature, email_verification_feature, override)
                 return ThirdPartyRecipe.__instance
-            else:
-                raise_general_exception(None, 'ThirdParty recipe has already been initialised. Please check your '
-                                              'code for bugs.')
+            raise_general_exception('ThirdParty recipe has already been initialised. Please check your code for bugs.')
 
         return func
 
@@ -137,7 +124,6 @@ class ThirdPartyRecipe(RecipeModule):
         if ThirdPartyRecipe.__instance is not None:
             return ThirdPartyRecipe.__instance
         raise_general_exception(
-            None,
             'Initialisation not done. Did you forget to call the SuperTokens.init function?')
 
     @staticmethod
@@ -145,13 +131,13 @@ class ThirdPartyRecipe(RecipeModule):
         if ('SUPERTOKENS_ENV' not in environ) or (
                 environ['SUPERTOKENS_ENV'] != 'testing'):
             raise_general_exception(
-                None, 'calling testing function in non testing env')
+                'calling testing function in non testing env')
         ThirdPartyRecipe.__instance = None
 
     # instance functions below...............
 
-    async def get_email_for_user_id(self, user_id: str) -> str:
-        user_info = await self.recipe_implementation.get_user_by_id(user_id)
+    async def get_email_for_user_id(self, user_id: str, user_context: Dict[str, Any]) -> str:
+        user_info = await self.recipe_implementation.get_user_by_id(user_id, user_context)
         if user_info is None:
             raise Exception('Unknown User ID provided')
         return user_info.email
