@@ -13,49 +13,63 @@
 # under the License.
 
 import json
+from typing import Any, Dict, Union
 
 from _pytest.fixtures import fixture
-from flask import Flask, jsonify, make_response, request
-from supertokens_python.recipe import emailpassword
-from supertokens_python.recipe.emailpassword.interfaces import APIOptions
-
-from supertokens_python import init, SupertokensConfig, InputAppInfo
+from flask import Flask, g, jsonify, make_response, request
+from supertokens_python import InputAppInfo, SupertokensConfig, init
 from supertokens_python.framework.flask import Middleware
-from supertokens_python.recipe import session
+from supertokens_python.recipe import emailpassword, session
+from supertokens_python.recipe.emailpassword.interfaces import (APIInterface,
+                                                                APIOptions)
+from supertokens_python.recipe.session import SessionContainer
 from supertokens_python.recipe.session.framework.flask import verify_session
-from supertokens_python.recipe.session.syncio import create_new_session, refresh_session, get_session, revoke_session
+from supertokens_python.recipe.session.syncio import (create_new_session,
+                                                      get_session,
+                                                      refresh_session,
+                                                      revoke_session)
 from tests.Flask.utils import extract_all_cookies
-from tests.utils import set_key_value_in_config, TEST_COOKIE_SAME_SITE_CONFIG_KEY, TEST_ACCESS_TOKEN_MAX_AGE_CONFIG_KEY, \
-    TEST_ACCESS_TOKEN_MAX_AGE_VALUE, TEST_ACCESS_TOKEN_PATH_CONFIG_KEY, TEST_ACCESS_TOKEN_PATH_VALUE, \
-    TEST_COOKIE_DOMAIN_CONFIG_KEY, TEST_COOKIE_DOMAIN_VALUE, TEST_REFRESH_TOKEN_MAX_AGE_CONFIG_KEY, \
-    TEST_REFRESH_TOKEN_MAX_AGE_VALUE, TEST_REFRESH_TOKEN_PATH_CONFIG_KEY, TEST_REFRESH_TOKEN_PATH_KEY_VALUE, \
-    TEST_COOKIE_SECURE_CONFIG_KEY, TEST_DRIVER_CONFIG_COOKIE_DOMAIN, \
-    TEST_DRIVER_CONFIG_ACCESS_TOKEN_PATH, TEST_DRIVER_CONFIG_REFRESH_TOKEN_PATH, TEST_DRIVER_CONFIG_COOKIE_SAME_SITE, \
-    start_st, reset, clean_st, setup_st
+from tests.utils import (TEST_ACCESS_TOKEN_MAX_AGE_CONFIG_KEY,
+                         TEST_ACCESS_TOKEN_MAX_AGE_VALUE,
+                         TEST_ACCESS_TOKEN_PATH_CONFIG_KEY,
+                         TEST_ACCESS_TOKEN_PATH_VALUE,
+                         TEST_COOKIE_DOMAIN_CONFIG_KEY,
+                         TEST_COOKIE_DOMAIN_VALUE,
+                         TEST_COOKIE_SAME_SITE_CONFIG_KEY,
+                         TEST_COOKIE_SECURE_CONFIG_KEY,
+                         TEST_DRIVER_CONFIG_ACCESS_TOKEN_PATH,
+                         TEST_DRIVER_CONFIG_COOKIE_DOMAIN,
+                         TEST_DRIVER_CONFIG_COOKIE_SAME_SITE,
+                         TEST_DRIVER_CONFIG_REFRESH_TOKEN_PATH,
+                         TEST_REFRESH_TOKEN_MAX_AGE_CONFIG_KEY,
+                         TEST_REFRESH_TOKEN_MAX_AGE_VALUE,
+                         TEST_REFRESH_TOKEN_PATH_CONFIG_KEY,
+                         TEST_REFRESH_TOKEN_PATH_KEY_VALUE, clean_st, reset,
+                         set_key_value_in_config, setup_st, start_st)
 
 
-def setup_function(f):
+def setup_function(_):
     reset()
     clean_st()
     setup_st()
 
 
-def teardown_function(f):
+def teardown_function(_):
     reset()
     clean_st()
 
 
 @fixture(scope='function')
 def driver_config_app():
-    def override_email_password_apis(original_implementation):
+    def override_email_password_apis(original_implementation: APIInterface):
 
         original_func = original_implementation.email_exists_get
 
-        async def email_exists_get(email: str, api_options: APIOptions):
+        async def email_exists_get(email: str, api_options: APIOptions, user_context: Dict[str, Any]):
             response_dict = {'custom': True}
             api_options.response.set_status_code(203)
             api_options.response.set_json_content(response_dict)
-            return await original_func(email, api_options)
+            return await original_func(email, api_options, user_context)
 
         original_implementation.email_exists_get = email_exists_get
         return original_implementation
@@ -84,50 +98,64 @@ def driver_config_app():
         )]
     )
 
-    @app.route('/test')
-    def t():
+    @app.route('/test')  # type: ignore
+    def t():  # type: ignore
         return jsonify({})
 
-    @app.route('/login')
-    def login():
+    @app.route('/login')  # type: ignore
+    def login():  # type: ignore
         user_id = 'userId'
         create_new_session(request, user_id, {}, {})
 
         return jsonify({'userId': user_id, 'session': 'ssss'})
 
-    @app.route('/refresh', methods=['POST'])
-    def custom_refresh():
+    @app.route('/refresh', methods=['POST'])  # type: ignore
+    def custom_refresh():  # type: ignore
         response = make_response(jsonify({}))
         refresh_session(request)
         return response
 
-    @app.route('/info', methods=['GET', 'OPTIONS'])
-    def custom_info():
-        if request.method == 'OPTIONS':
+    @app.route('/info', methods=['GET', 'OPTIONS'])  # type: ignore
+    def custom_info():  # type: ignore
+        if request.method == 'OPTIONS':  # type: ignore
             return jsonify({'method': 'option'})
         response = make_response(jsonify({}))
         get_session(request, True)
         return response
 
-    @app.route('/handle', methods=['GET', 'OPTIONS'])
-    def custom_handle_api():
-        if request.method == 'OPTIONS':
+    @app.route('/handle', methods=['GET', 'OPTIONS'])  # type: ignore
+    def custom_handle_api():  # type: ignore
+        if request.method == 'OPTIONS':  # type: ignore
             return jsonify({'method': 'option'})
-        session = get_session(request, True)
-        return jsonify({'s': session['user_id']})
+        session: Union[None, SessionContainer] = get_session(request, True)
+        if session is None:
+            raise Exception("Should never come here")
+        return jsonify({'s': session.get_user_id()})
 
-    @app.route('/logout', methods=['POST'])
+    @app.route('/handle-session-optional', methods=['GET', 'OPTIONS'])  # type: ignore
     @verify_session(session_required=False)
-    def custom_logout():
+    def optional_session():  # type: ignore
+        if request.method == 'OPTIONS':  # type: ignore
+            return jsonify({'method': 'option'})
+        session: Union[SessionContainer, None] = g.supertokens  # type: ignore
+        if session is None:
+            return jsonify({'s': "empty session"})
+        return jsonify({'s': session.get_user_id()})
+
+    @app.route('/logout', methods=['POST'])  # type: ignore
+    @verify_session(session_required=False)
+    def custom_logout():  # type: ignore
         response = make_response(jsonify({}))
-        supertokens_session = get_session(request, False)
-        revoke_session(supertokens_session['user_id'])
+        session: Union[None, SessionContainer] = get_session(request, True)
+        if session is None:
+            raise Exception("Should never come here")
+        revoke_session(session.get_user_id())
         return response
 
     return app
 
 
-def test_cookie_login_and_refresh(driver_config_app):
+def test_cookie_login_and_refresh(driver_config_app: Any):
     start_st()
 
     set_key_value_in_config(
@@ -150,7 +178,7 @@ def test_cookie_login_and_refresh(driver_config_app):
         TEST_REFRESH_TOKEN_PATH_KEY_VALUE)
     set_key_value_in_config(
         TEST_COOKIE_SECURE_CONFIG_KEY,
-        False)
+        "false")
 
     response_1 = driver_config_app.test_client().get('/login')
     cookies_1 = extract_all_cookies(response_1)
@@ -165,9 +193,12 @@ def test_cookie_login_and_refresh(driver_config_app):
     assert cookies_1['sAccessToken']['httponly']
     assert cookies_1['sRefreshToken']['httponly']
     assert cookies_1['sIdRefreshToken']['httponly']
-    assert cookies_1['sAccessToken']['samesite'].lower() == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
-    assert cookies_1['sRefreshToken']['samesite'].lower() == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
-    assert cookies_1['sIdRefreshToken']['samesite'].lower() == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
+    assert cookies_1['sAccessToken']['samesite'].lower(
+    ) == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
+    assert cookies_1['sRefreshToken']['samesite'].lower(
+    ) == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
+    assert cookies_1['sIdRefreshToken']['samesite'].lower(
+    ) == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
 
     test_client = driver_config_app.test_client()
     test_client.set_cookie(
@@ -194,12 +225,15 @@ def test_cookie_login_and_refresh(driver_config_app):
     assert cookies_2['sAccessToken']['httponly']
     assert cookies_2['sRefreshToken']['httponly']
     assert cookies_2['sIdRefreshToken']['httponly']
-    assert cookies_2['sAccessToken']['samesite'].lower() == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
-    assert cookies_2['sRefreshToken']['samesite'].lower() == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
-    assert cookies_2['sIdRefreshToken']['samesite'].lower() == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
+    assert cookies_2['sAccessToken']['samesite'].lower(
+    ) == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
+    assert cookies_2['sRefreshToken']['samesite'].lower(
+    ) == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
+    assert cookies_2['sIdRefreshToken']['samesite'].lower(
+    ) == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
 
 
-def test_login_refresh_no_csrf(driver_config_app):
+def test_login_refresh_no_csrf(driver_config_app: Any):
     start_st()
 
     set_key_value_in_config(
@@ -222,7 +256,7 @@ def test_login_refresh_no_csrf(driver_config_app):
         TEST_REFRESH_TOKEN_PATH_KEY_VALUE)
     set_key_value_in_config(
         TEST_COOKIE_SECURE_CONFIG_KEY,
-        False)
+        "false")
 
     response_1 = driver_config_app.test_client().get('/login')
     cookies_1 = extract_all_cookies(response_1)
@@ -237,9 +271,12 @@ def test_login_refresh_no_csrf(driver_config_app):
     assert cookies_1['sAccessToken']['httponly']
     assert cookies_1['sRefreshToken']['httponly']
     assert cookies_1['sIdRefreshToken']['httponly']
-    assert cookies_1['sAccessToken']['samesite'].lower() == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
-    assert cookies_1['sRefreshToken']['samesite'].lower() == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
-    assert cookies_1['sIdRefreshToken']['samesite'].lower() == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
+    assert cookies_1['sAccessToken']['samesite'].lower(
+    ) == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
+    assert cookies_1['sRefreshToken']['samesite'].lower(
+    ) == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
+    assert cookies_1['sIdRefreshToken']['samesite'].lower(
+    ) == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
 
     test_client = driver_config_app.test_client()
     test_client.set_cookie(
@@ -261,7 +298,7 @@ def test_login_refresh_no_csrf(driver_config_app):
     assert result.status_code == 401
 
 
-def test_login_logout(driver_config_app):
+def test_login_logout(driver_config_app: Any):
     start_st()
 
     set_key_value_in_config(
@@ -284,7 +321,7 @@ def test_login_logout(driver_config_app):
         TEST_REFRESH_TOKEN_PATH_KEY_VALUE)
     set_key_value_in_config(
         TEST_COOKIE_SECURE_CONFIG_KEY,
-        False)
+        "false")
 
     response_1 = driver_config_app.test_client().get('/login')
     cookies_1 = extract_all_cookies(response_1)
@@ -299,9 +336,12 @@ def test_login_logout(driver_config_app):
     assert cookies_1['sAccessToken']['httponly']
     assert cookies_1['sRefreshToken']['httponly']
     assert cookies_1['sIdRefreshToken']['httponly']
-    assert cookies_1['sAccessToken']['samesite'].lower() == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
-    assert cookies_1['sRefreshToken']['samesite'].lower() == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
-    assert cookies_1['sIdRefreshToken']['samesite'].lower() == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
+    assert cookies_1['sAccessToken']['samesite'].lower(
+    ) == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
+    assert cookies_1['sRefreshToken']['samesite'].lower(
+    ) == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
+    assert cookies_1['sIdRefreshToken']['samesite'].lower(
+    ) == TEST_DRIVER_CONFIG_COOKIE_SAME_SITE
 
     test_client = driver_config_app.test_client()
     test_client.set_cookie(
@@ -320,7 +360,7 @@ def test_login_logout(driver_config_app):
                                   )
 
     cookies_2 = extract_all_cookies(response_2)
-    assert cookies_2 == {}
+    assert not cookies_2
 
     response_3 = test_client.post('/logout',
                                   headers={
@@ -331,7 +371,7 @@ def test_login_logout(driver_config_app):
     assert response_3.status_code == 200
 
 
-def test_login_handle(driver_config_app):
+def test_login_handle(driver_config_app: Any):
     start_st()
 
     set_key_value_in_config(
@@ -354,7 +394,7 @@ def test_login_handle(driver_config_app):
         TEST_REFRESH_TOKEN_PATH_KEY_VALUE)
     set_key_value_in_config(
         TEST_COOKIE_SECURE_CONFIG_KEY,
-        False)
+        "false")
 
     response_1 = driver_config_app.test_client().get('/login')
     cookies_1 = extract_all_cookies(response_1)
@@ -378,12 +418,25 @@ def test_login_handle(driver_config_app):
     assert "s" in response_dict
 
 
-def test_custom_response(driver_config_app):
+def test_custom_response(driver_config_app: Any):
     start_st()
 
     test_client = driver_config_app.test_client()
-    response = test_client.get('/auth/signup/email/exists?email=test@example.com')
+    response = test_client.get(
+        '/auth/signup/email/exists?email=test@example.com')
 
     dict_response = json.loads(response.data)
     assert response.status_code == 203
     assert dict_response["custom"]
+
+
+def test_optional_session(driver_config_app: Any):
+    start_st()
+
+    test_client = driver_config_app.test_client()
+    response = test_client.get(
+        '/handle-session-optional')
+
+    dict_response = json.loads(response.data)
+    assert response.status_code == 200
+    assert dict_response["s"] == "empty session"
